@@ -4,13 +4,21 @@ Widget::Widget(QWidget *parent)
     : QWidget(parent)
 {
     megaphoneActive = false;
-    transmitterActive = false; //check
+    transmitterActive = (getFMTxProperties("state").toString() == "enabled") ? true : false;
+    lastVolumeValue = 100;
+
+    setGeometry(0, 0, 800, 480);
 
     QDBusConnection::systemBus().connect("com.nokia.FMTx",
                                          "/com/nokia/fmtx/default",
                                          "com.nokia.policy",
                                          "info", this,
                                          SLOT(onTransmitterChanged(QString, const QString &, QStringList)));
+    QDBusConnection::systemBus().connect("com.nokia.FMTx",
+                                         "/com/nokia/fmtx/default",
+                                         "com.nokia.FMTx.Device",
+                                         "Changed", this,
+                                         SLOT(onFMTxPropChanged()));
     QDBusConnection::systemBus().connect("com.nokia.csd.Call",
                                          "/com/nokia/csd/call",
                                          "com.nokia.csd.Call",
@@ -23,26 +31,45 @@ Widget::Widget(QWidget *parent)
                                          "Created",
                                          this, SLOT(callIncoming()));
 
-    freqLabel = new QLabel("Freq:\n" + QString::number(getFrequency()/1000.0) + "MHz", this);
-    freqLabel->setStyleSheet("font:18pt;"
-                             "qproperty-alignment: AlignCenter;");
-    freqLabel->setGeometry(0, 180, 800, 60);
+    freqLabel = new QPushButton(this);
+    freqLabel->setStyleSheet("font:18pt;");
+    freqLabel->setGeometry(10, 140, 780, 80);
+    freqLabel->setFlat(true);
+
+    connect(freqLabel, SIGNAL(clicked()), this, SLOT(onFreqLabelClicked()));
+
+    _freq = getFMTxProperties("frequency").toUInt();
+    setFreqLabel(_freq);
 
     button1 = new QPushButton("Enable megaphone", this);
-    button1->setGeometry(0, 30, 350, 370);
+    button1->setGeometry(10, 40, 320, 260);
     connect(button1, SIGNAL(clicked()), this, SLOT(toggleMegaphone()));
 
     button2 = new QPushButton("Enable FM transmitter", this);
-    button2->setGeometry(450, 30, 350, 370);
+    button2->setGeometry(470, 40, 320, 260);
     connect(button2, SIGNAL(clicked()), this, SLOT(toggleTransmitter()));
+
+    volumeSlider = new QSlider(Qt::Horizontal, this);
+    volumeSlider->setGeometry(30, 350, 740, 50);
+    volumeSlider->setMinimum(0);
+    volumeSlider->setMaximum(100);
+    volumeSlider->setValue(100);
+    connect(volumeSlider, SIGNAL(valueChanged(int)), this, SLOT(onVolumeSliderChanged(int)));
+
+    uint freq_min, freq_max;
+    getFreqRange(freq_min, freq_max);
+
+    freqWidget = new FreqWidget(freq_min, freq_max, _freq, this);
+    freqWidget->hide();
+
+    connect(freqWidget, SIGNAL(freqChanged(uint)), this, SLOT(setFrequency(uint)));
+    connect(freqWidget, SIGNAL(clicked()), freqWidget, SLOT(hide()));
 }
 
 Widget::~Widget()
 {
-    if(megaphoneActive) {
-       gst_element_set_state(pipeline, GST_STATE_NULL);
-       gst_object_unref(pipeline);
-    }
+    if(megaphoneActive)
+       disableMegaphone();
 }
 
 void Widget::toggleMegaphone()
@@ -52,10 +79,7 @@ void Widget::toggleMegaphone()
     else
         megaphoneActive = enableMegaphone();
 
-    if(megaphoneActive)
-        button1->setText("Disable megaphone");
-    else
-        button1->setText("Enable megaphone");
+    button1->setText(megaphoneActive ? "Disable megaphone" : "Enable megaphone");
 }
 
 bool Widget::enableMegaphone()
@@ -63,7 +87,7 @@ bool Widget::enableMegaphone()
     gst_init(NULL, NULL);
 
     pipeline = gst_pipeline_new("pipeline");
-    GstElement *source, *sink;
+    GstElement *sink;
     source = gst_element_factory_make("pulsesrc", "source");
     sink = gst_element_factory_make("pulsesink", "sink");
 
@@ -76,65 +100,24 @@ bool Widget::enableMegaphone()
         return false;
 
     gst_element_set_state(pipeline, GST_STATE_PLAYING);
-
     return true;
 }
 
 bool Widget::disableMegaphone()
 {
+    setVolume(100);
+    volumeSlider->setValue(100);
     gst_element_set_state(pipeline, GST_STATE_NULL);
     gst_object_unref(pipeline);
     return false;
 }
 
-void Widget::enableTransmitter() //http://maemo.org/api_refs/5.0/5.0-final/fmtx-middleware/fmtx_api.html
-{
-    QDBusMessage msg = QDBusMessage::createMethodCall("com.nokia.FMTx",
-                                                      "/com/nokia/fmtx/default",
-                                                      "org.freedesktop.DBus.Properties",
-                                                      "Set");
-
-    QVariantList list;
-    list.append("com.nokia.FMTx.device");
-    list.append("state");
-    list.append(QVariant::fromValue(QDBusVariant("enabled")));
-    msg.setArguments(list);
-    QDBusConnection::systemBus().asyncCall(msg);
-}
-
-void Widget::disableTransmitter()
-{
-    QDBusMessage msg = QDBusMessage::createMethodCall("com.nokia.FMTx",
-                                                      "/com/nokia/fmtx/default",
-                                                      "org.freedesktop.DBus.Properties",
-                                                      "Set");
-    QVariantList list;
-    list.append("com.nokia.FMTx.device");
-    list.append("state");
-    list.append(QVariant::fromValue(QDBusVariant("disabled")));
-    msg.setArguments(list);
-    QDBusConnection::systemBus().asyncCall(msg);
-}
-
 void Widget::toggleTransmitter()
 {
     if(transmitterActive)
-        disableTransmitter();
+        setFMTxProperties("state", QDBusVariant("disabled"));
     else
-        enableTransmitter();
-}
-
-void Widget::onTransmitterChanged(QString , const QString &msg, QStringList)
-{
-    transmitterActive = msg.toInt();
-
-    if(transmitterActive)
-    {
-        button2->setText("Disable FM transmitter");
-        setFreqLabel();
-    }
-    else
-        button2->setText("Enable FM transmitter");
+        setFMTxProperties("state", QDBusVariant("enabled"));
 }
 
 void Widget::callIncoming()
@@ -146,7 +129,26 @@ void Widget::callIncoming()
     }
 }
 
-uint Widget::getFrequency()
+void Widget::setFrequency(const uint &freq)
+{
+    setFMTxProperties("frequency", QDBusVariant(freq));
+}
+
+void Widget::setFMTxProperties(const QString &prop, const QDBusVariant &value) //http://maemo.org/api_refs/5.0/5.0-final/fmtx-middleware/fmtx_api.html
+{
+    QDBusMessage msg = QDBusMessage::createMethodCall("com.nokia.FMTx",
+                                                      "/com/nokia/fmtx/default",
+                                                      "org.freedesktop.DBus.Properties",
+                                                      "Set");
+    QVariantList list;
+    list.append("com.nokia.FMTx.device");
+    list.append(prop);
+    list.append(QVariant::fromValue(value));
+    msg.setArguments(list);
+    QDBusConnection::systemBus().asyncCall(msg);
+}
+
+QVariant Widget::getFMTxProperties(const QString &prop)
 {
     QDBusMessage msg = QDBusMessage::createMethodCall("com.nokia.FMTx",
                                                       "/com/nokia/fmtx/default",
@@ -154,17 +156,75 @@ uint Widget::getFrequency()
                                                       "Get");
     QVariantList list;
     list.append("com.nokia.FMTx.device");
-    list.append("frequency");
+    list.append(prop);
     msg.setArguments(list);
     QDBusReply<QVariant> reply = QDBusConnection::systemBus().call(msg);
-
     if(reply.isValid())
-        return QVariant(reply.value()).toUInt();
-    else
-        return 88;
+        return reply.value();
+    else return QVariant();
+}
+void Widget::getFreqRange(uint &freq_min, uint &freq_max)
+{
+    freq_min = getFMTxProperties("freq_min").toUInt();
+    freq_max = getFMTxProperties("freq_max").toUInt();
+
+    qDebug() << "freq_min: " << freq_min << " freq_max: " << freq_max;
 }
 
-void Widget::setFreqLabel()
+void Widget::setFreqLabel(const uint &freq)
 {
-    freqLabel->setText("Freq:\n" + QString::number(getFrequency()/1000.0) + "MHz");
+    freqLabel->setText("Freq:\n" + QString::number(freq/1000) + "." + QString::number((freq%1000)/10)  + "MHz");
 }
+
+void Widget::setVolume(const int &percent)
+{
+    QtConcurrent::run(this, &Widget::setVolumeThread, percent);
+}
+
+void Widget::setVolumeThread(const int &percent)
+{
+    if(percent <= 100 && percent >= 0)
+    {
+        QProcess proc;
+        proc.start(QString("amixer set Capture %1%").arg(percent));
+        proc.waitForFinished();
+    }
+}
+
+
+void Widget::onVolumeSliderChanged(const int &value)
+{
+    if(value > 95)
+    {
+        setVolume(100);
+        lastVolumeValue = 100;
+    }
+    else if(lastVolumeValue + 5 <= value || lastVolumeValue - 5 >= value)
+    {
+        setVolume(value);
+        lastVolumeValue = value;
+    }
+}
+
+void Widget::onFreqLabelClicked()
+{
+    freqWidget->setFreq(_freq);
+    freqWidget->show();
+}
+
+void Widget::onTransmitterChanged(QString , const QString &msg, QStringList)
+{
+    transmitterActive = msg.toInt();
+
+    if(transmitterActive)
+        button2->setText("Disable FM transmitter");
+    else
+        button2->setText("Enable FM transmitter");
+}
+
+void Widget::onFMTxPropChanged()
+{
+    _freq = getFMTxProperties("frequency").toUInt();
+    setFreqLabel(_freq);
+}
+
